"""RGB LED strip as a full color light."""

import circuitmatter as cm
from circuitmatter.device_types.lighting import extended_color

import board
import neopixel


class RGBPixel(extended_color.ExtendedColorLight):
    def __init__(self, name, pixels):
        super().__init__(name)
        self._pixels = pixels
        self._brightness = 0.1

    @property
    def color_rgb(self):
        return self._color

    @color_rgb.setter
    def color_rgb(self, value):
        self._pixels.fill(value)
        print(f"new color 0x{value:06x}")

    @property
    def brightness(self):
        return self._brightness

    @brightness.setter
    def brightness(self, value):
        self._brightness = value
        self._pixels.brightness = value
        print(f"new brightness {value}")

    def on(self):
        self._pixels.brightness = self._brightness
        print("on!")

    def off(self):
        self._pixels.brightness = 0
        print("off!")


matter = cm.CircuitMatter()
# This is a 8mm NeoPixel breadboard LED. (https://www.adafruit.com/product/1734)
# Any pixelbuf compatible strip should work. The RGBPixel class will control the
# entire strip of pixels.
np = neopixel.NeoPixel(board.D12, 1, pixel_order="RGB")
led = RGBPixel("led1", np)
matter.add_device(led)
while True:
    matter.process_packets()



import time

from . import interaction_model
from .exchange import Exchange
from .protocol import ProtocolId


class Subscription:
    def __init__(self, _id, session, min_interval, max_interval):
        self.id = _id
        session.subscriptions[self.id] = self
        self.active = True
        self._reports = []
        self._session = session
        self._min_interval = min_interval
        self._max_interval = max_interval
        # Initial transmit is handled during the subscription call.
        self._last_transmit = time.monotonic()

    def send_reports(self, exchange=None):
        time_since = time.monotonic() - self._last_transmit
        if time_since < self._min_interval:
            return
        if not self._reports and time_since < (self._max_interval - 1):
            return

        # create a new exchange and send reports.
        exchange = Exchange(self._session, [ProtocolId.INTERACTION_MODEL])

        response = interaction_model.ReportDataMessage()
        response.SubscriptionId = self.id
        response.AttributeReports = self._reports
        if not self._reports:
            # No response on empty reports
            response.SuppressResponse = True
        exchange.send(response)
        if not self._reports:
            exchange.close()
        print(
            "reporting",
            self._reports,
            self._min_interval,
            time_since,
            self._max_interval,
        )
        # Use a new list so we don't clear the one we're sending.
        self._reports = []
        self._last_transmit = time.monotonic()

    def append_report(self, report):
        self._reports.append(report)

    def ack_report(self):
        pass
